#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include "installer.h"

#define DENTRY_SLOTS 16
static const char* dentries[DENTRY_SLOTS] = {0};
static int last_dentry = 0;

static void showmode(char type, int mode)
{
  putchar(type);
  putchar((mode & 0400) ? 'r' : '-');
  putchar((mode & 0200) ? 'w' : '-');
  putchar((mode & 0100) ?
	  (mode & 04000) ? 's' : 'x' :
	  (mode & 04000) ? 'S' : '-');
  putchar((mode & 040) ? 'r' : '-');
  putchar((mode & 020) ? 'w' : '-');
  putchar((mode & 010) ?
	  (mode & 02000) ? 's' : 'x' :
	  (mode & 02000) ? 'S' : '-');
  putchar((mode & 04) ? 'r' : '-');
  putchar((mode & 02) ? 'w' : '-');
  putchar((mode & 01) ?
	  (mode & 01000) ? 't' : 'x' :
	  (mode & 01000) ? 'T' : '-');
}

static void show(char type,
		 int dir, const char* subdir, const char* filename,
		 int uid, int gid, int mode)
{
  showmode(type, mode);

  if (uid == -1) fputs("      N/A", stdout);
  else printf(" %8d", uid);
  if (gid == -1) fputs("      N/A", stdout);
  else printf(" %8d", gid);

  putchar(' ');
  fputs(dentries[dir], stdout);
  putchar('/');
  if (subdir) {
    fputs(subdir, stdout);
    putchar('/');
  }
  puts(filename);
}

void c(int dir, const char* subdir, const char* filename,
       int uid, int gid, int mode)
{
  show('-', dir, subdir, filename, uid, gid, mode);
}

void d(int dir, const char* subdir,
       int uid, int gid, int mode)
{
  show('d', dir, 0, subdir, uid, gid, mode);
}

int opendir(const char* dir)
{
  if (last_dentry >= DENTRY_SLOTS) {
    fputs("instshow: error: Too many open directories!\n", stderr);
    exit(1);
  }
  dentries[last_dentry] = dir;
  return last_dentry++;
}

int main(void)
{
  puts(" type/mode    owner    group path");
  insthier();
  return 0;
}
